import { NetworkMonitor, NetworkMonitorConfig, ResourceHealth, SystemHealthChangedEvent, SystemHealthStatus } from '@genesislcap/foundation-comms';
import { ElementStyles } from '@microsoft/fast-element';
import type { ElementDefinitionContext, FoundationElementDefinition } from '@microsoft/fast-foundation';
import { FoundationElement } from '@microsoft/fast-foundation';
import { LogLevel } from '../../utils';
import { SystemHealthOverlayMode, TimeTillNextUpdate } from './types';
/**
 * SystemHealthOverlay
 * @public
 */
export declare class SystemHealthOverlay extends FoundationElement {
    /**
     * High-level configuration.
     * @internal
     */
    protected config: NetworkMonitorConfig;
    /**
     * @privateRemarks
     * Including a reference allows this overlay component to manage the NetworkMonitor's create/destroy lifecycle,
     * so that users only need to include this overlay in their markup. ie. if you're adding the overlay, you expect the
     * NetworkMonitor to be created to power it. Users may also inject the NetworkMonitor in their main class to create it,
     * as multiple calls to create will be ignored.
     * @internal
     */
    protected networkMonitor: NetworkMonitor;
    resourceList: ResourceHealth[];
    isOpen: boolean;
    isOpenChanged(): void;
    isDismissed: boolean;
    isDismissedChanged(): void;
    templateClassNames: string;
    containerClassNames: string;
    isOnlineClassNames: string;
    isHealthyClassNames: string;
    isSocketHealthyClassNames: string;
    resourcesAllConnectedClassNames: string;
    timeTillNextUpdate: TimeTillNextUpdate;
    protected timeTillNextUpdateInterval: ReturnType<typeof setInterval>;
    /**
     * @example
     * App authors can set this property on the component to indicate they have a save data callback.
     * ```html
     * <foundation-system-health-overlay :saveDataCallback=${(x) => x.onSaveAppData}>
     * ```
     */
    saveDataCallback: () => void;
    /**
     * Controls the overlay mode, either app or component level.
     * @remarks
     * HTML attribute: mode
     */
    mode: SystemHealthOverlayMode;
    protected modeChanged(): void;
    /**
     * Allows users to control the logging level of the network monitor via the component.
     * @remarks
     * HTML attribute: log-level
     * @privateRemarks
     * Might be nicer to have users set these as 'error', 'debug' etc. and map them.
     */
    logLevel: LogLevel;
    protected logLevelChanged(): void;
    /**
     * Last known system health status.
     * @privateRemarks
     * This will be updated momentarily when NetworkMonitor.create is called.
     * We may want to delay opening slightly to avoid any overlay flashing during initial resource loading. Noticed at
     * times we may get a temporary health issue on a resource with a NACK stating 'Dataserver subscription does not exist'
     * or similar, which will cause the re-datalogon to occur to try and recover the resource. This might indicate clean
     * up issues elsewhere that need to be addressed.
     */
    status: SystemHealthStatus;
    protected statusChanged(prev: SystemHealthStatus, next: SystemHealthStatus): void;
    protected updateClassNames(): void;
    protected updateTemplateClassNames(): void;
    protected startTimeTillNextUpdateCountdown(): void;
    connectedCallback(): void;
    disconnectedCallback(): void;
    onSystemHealthChanged: (event: SystemHealthChangedEvent) => void;
    onOpen: () => void;
    onClose: () => void;
    onDismiss: () => void;
    onUpdateStatus: () => void;
    onReportStatus: () => void;
    onReloadWindow: () => void;
    onDocumentKeydown: (e: KeyboardEvent) => void;
    get isSocketHealthy(): boolean;
    get validResourceList(): boolean;
    /**
     * @internal
     */
    protected eventBridge(): HTMLElement | Document;
    /**
     * @internal
     */
    protected addEventListeners(): void;
    /**
     * @internal
     */
    protected removeEventListeners(): void;
    static getCheckmarkTemplate(isGood: boolean): import("@microsoft/fast-element").ViewTemplate<any, any>;
}
/**
 * Foundation system health overlay template
 *
 * @public
 */
export declare const foundationSystemHealthOverlayTemplate: (context: ElementDefinitionContext, definition: FoundationElementDefinition) => import("@microsoft/fast-element").ViewTemplate<SystemHealthOverlay, any>;
/**
 * Foundation system health overlay styles
 * @public
 */
export declare const foundationSystemHealthOverlayStyles: (context: ElementDefinitionContext, definition: FoundationElementDefinition) => ElementStyles;
/**
 * Foundation system health overlay styles
 * @public
 */
export declare const foundationSystemHealthOverlayShadowOptions: ShadowRootInit;
/**
 * Foundation system health overlay styles
 * @public
 */
export declare const defaultSystemHealthOverlayConfig: {};
/**
 * Foundation system health overlay
 *
 * @public
 * @remarks
 * HTML Element: \<foundation-system-health-overlay\>
 */
export declare const foundationSystemHealthOverlay: (overrideDefinition?: import("@microsoft/fast-foundation").OverrideFoundationElementDefinition<FoundationElementDefinition>) => import("@microsoft/fast-foundation").FoundationElementRegistry<FoundationElementDefinition, typeof SystemHealthOverlay>;
//# sourceMappingURL=system-health-overlay.d.ts.map