import { __decorate } from "tslib";
import { attr, observable, volatile } from '@microsoft/fast-element';
import { FoundationElement } from '@microsoft/fast-foundation';
import { Position } from '../multiselect';
import { wasClickOutsideElement } from '../utils';
import { categorizedMultiselectStyles as styles } from './categorized-multiselect.styles';
import { categorizedMultiselectTemplate as template } from './categorized-multiselect.template';
export const foundationCategorizedMultiselectShadowOptions = undefined;
export const defaultCategorizedMultiselectConfig = {};
/**
 * @tagname %%prefix%%-categorized-multiselect
 */
export class CategorizedMultiselect extends FoundationElement {
    constructor() {
        super(...arguments);
        this.name = '';
        this.autoPosition = true;
        this.search = true;
        this.disabled = false;
        this.clickOutside = (e) => this.handleClickOutside(e);
        this.open = false;
        this.options = [];
        this.selectedOptions = [];
        /**
         * Controls the positioning of the multiselect dropdown. Accepts the following values:
         * - Position.ABOVE: Forces the dropdown to always open above the input field.
         * - Position.BELOW: Forces the dropdown to always open below the input field.
         */
        this.position = Position.BELOW;
    }
    openChanged(previous, open) {
        var _a, _b;
        if (open) {
            document.addEventListener('mousedown', this.clickOutside);
            (_a = this.optionsElement) === null || _a === void 0 ? void 0 : _a.classList.add('visible');
        }
        else {
            document.removeEventListener('mousedown', this.clickOutside);
            (_b = this.optionsElement) === null || _b === void 0 ? void 0 : _b.classList.remove('visible');
        }
    }
    connectedCallback() {
        var _a;
        super.connectedCallback();
        if (this.position === Position.ABOVE) {
            (_a = this.optionsElement) === null || _a === void 0 ? void 0 : _a.classList.add('position-above');
        }
    }
    disconnectedCallback() {
        super.disconnectedCallback();
        document.removeEventListener('mousedown', this.clickOutside);
    }
    toggle() {
        if (!this.open) {
            this.setDropdownPosition();
        }
        this.open = !this.open;
    }
    sortFilteredOptions(options) {
        const sortedData = (options !== null && options !== void 0 ? options : this.options).reduce((acc, curr) => {
            const type = curr.type || 'Other';
            if (!acc[type]) {
                acc[type] = {
                    type: type,
                    options: [curr],
                };
            }
            else {
                acc[type].options.push(curr);
            }
            return acc;
        }, {});
        const result = Object.values(sortedData);
        return result;
    }
    optionsChanged() {
        this.filteredAvailableOptions = this.sortFilteredOptions();
    }
    /**
     * Sets the dropdown position based on the current configuration.
     * @remarks
     * If `autoPosition` is `true`, this method calculates the available space above and below the input field in the viewport and positions the dropdown accordingly.
     * This ensures the dropdown is always visible to the user. If `autoPosition` is `false`, the dropdown position is locked to either 'above' or 'below', as specified by the `position` attribute.
     * This method is intended for internal use by the component and is marked as private.
     * @private
     */
    setDropdownPosition() {
        let calculatedPosition = this.position;
        if (this.autoPosition) {
            let container = this.assignedSlot ? this.assignedSlot.parentElement : this.parentElement;
            while (container && !container.classList.contains('content-wrapper')) {
                container = container.parentElement;
            }
            if (!container) {
                const multiselectRect = this.getBoundingClientRect();
                const viewportHeight = window.innerHeight;
                const spaceAbove = multiselectRect.top;
                const spaceBelow = viewportHeight - multiselectRect.bottom;
                calculatedPosition =
                    spaceBelow < multiselectRect.height && spaceAbove > spaceBelow
                        ? Position.ABOVE
                        : Position.BELOW;
            }
            else {
                const containerRect = container.getBoundingClientRect();
                const thisRect = this.getBoundingClientRect();
                const midPoint = containerRect.top + containerRect.height / 2;
                calculatedPosition = thisRect.top < midPoint ? Position.BELOW : Position.ABOVE;
            }
        }
        if (calculatedPosition === Position.ABOVE) {
            this.optionsElement.classList.add('position-above');
            this.optionsElement.classList.remove('position-below');
        }
        else if (calculatedPosition === Position.BELOW) {
            this.optionsElement.classList.add('position-below');
            this.optionsElement.classList.remove('position-above');
        }
    }
    optionGetter(option) {
        return option.value;
    }
    getItemLabel(option) {
        let value;
        if (!option) {
            return '';
        }
        else {
            value = option.label || option.value || '';
        }
        return this.valueFormatter ? this.valueFormatter(value) : value;
    }
    isOptionDisabled(option) {
        if (!option || typeof option === 'string') {
            return false;
        }
        return option.disabled || false;
    }
    get getPlaceholder() {
        return `Search ${this.name}`.trim().concat('...');
    }
    isOptionSelected(normalizedOption) {
        return this.selectedOptions.includes(this.optionGetter(normalizedOption));
    }
    selectCheckboxOne(normalizedOption) {
        const option = this.optionGetter(normalizedOption);
        const isIncluded = this.selectedOptions.includes(option);
        if (!isIncluded) {
            this.selectedOptions = [...this.selectedOptions, option];
        }
        else {
            this.selectedOptions = [...this.selectedOptions];
            this.selectedOptions.splice(this.selectedOptions.indexOf(option), 1);
        }
        this.$emit('selectionChange', this.selectedOptions);
    }
    setFilter() {
        const filteredOptions = this.options.filter((option) => {
            var _a;
            const optionDescription = option.description;
            const optionValue = (_a = option.label) !== null && _a !== void 0 ? _a : option.value;
            if (optionDescription === null || optionDescription === void 0 ? void 0 : optionDescription.toLowerCase().includes(this.filterSearchValue.toLowerCase())) {
                return option;
            }
            if (optionValue.toLowerCase().includes(this.filterSearchValue.toLowerCase())) {
                return option;
            }
        });
        this.filteredAvailableOptions = this.sortFilteredOptions(filteredOptions);
    }
    changeHandler(event) {
        event.preventDefault();
        event.stopImmediatePropagation();
    }
    handleClickOutside(event) {
        if (wasClickOutsideElement(event, this)) {
            this.open = false;
        }
    }
}
__decorate([
    attr
], CategorizedMultiselect.prototype, "name", void 0);
__decorate([
    attr({ mode: 'boolean', attribute: 'auto-position' })
], CategorizedMultiselect.prototype, "autoPosition", void 0);
__decorate([
    attr({ mode: 'boolean' })
], CategorizedMultiselect.prototype, "search", void 0);
__decorate([
    attr({ mode: 'boolean' })
], CategorizedMultiselect.prototype, "disabled", void 0);
__decorate([
    observable
], CategorizedMultiselect.prototype, "defaultSlottedNodes", void 0);
__decorate([
    observable
], CategorizedMultiselect.prototype, "valueFormatter", void 0);
__decorate([
    observable
], CategorizedMultiselect.prototype, "filterSearchValue", void 0);
__decorate([
    observable
], CategorizedMultiselect.prototype, "open", void 0);
__decorate([
    observable
], CategorizedMultiselect.prototype, "options", void 0);
__decorate([
    observable
], CategorizedMultiselect.prototype, "selectedOptions", void 0);
__decorate([
    observable
], CategorizedMultiselect.prototype, "filteredAvailableOptions", void 0);
__decorate([
    observable
], CategorizedMultiselect.prototype, "position", void 0);
__decorate([
    volatile
], CategorizedMultiselect.prototype, "getPlaceholder", null);
export const foundationCategorizedMultiselect = CategorizedMultiselect.compose(Object.assign({ baseName: 'categorized-multiselect', template,
    styles, shadowOptions: foundationCategorizedMultiselectShadowOptions }, defaultCategorizedMultiselectConfig));
