import { __decorate } from "tslib";
import { networkLogger, NetworkMonitor, NetworkMonitorConfig, SystemHealthChanged, } from '@genesislcap/foundation-comms';
import { formatDateTimeTimestamp } from '@genesislcap/foundation-utils';
import { attr, css, html, nullableNumberConverter, observable, repeat, when, } from '@microsoft/fast-element';
import { FoundationElement } from '@microsoft/fast-foundation';
import { ArrowKeys, classNames, keyEscape, pascalCase } from '@microsoft/fast-web-utilities';
import { foundationProgress } from '../../progress';
import { provideDesignSystem } from '../../provide-design-system';
import { logger } from '../../utils';
import { SystemHealthOverlayMode } from './types';
/**
 * Referenced as used
 */
provideDesignSystem().register(foundationProgress({
    prefix: 'next-update',
}));
/**
 * SystemHealthOverlay
 * @public
 */
export class SystemHealthOverlay extends FoundationElement {
    constructor() {
        super(...arguments);
        this.timeTillNextUpdate = {
            totalMs: 10000,
            remainMs: 10000,
            remainPercent: 100,
        };
        /**
         * Controls the overlay mode, either app or component level.
         * @remarks
         * HTML attribute: mode
         */
        this.mode = SystemHealthOverlayMode.app;
        /**
         * Last known system health status.
         * @privateRemarks
         * This will be updated momentarily when NetworkMonitor.create is called.
         * We may want to delay opening slightly to avoid any overlay flashing during initial resource loading. Noticed at
         * times we may get a temporary health issue on a resource with a NACK stating 'Dataserver subscription does not exist'
         * or similar, which will cause the re-datalogon to occur to try and recover the resource. This might indicate clean
         * up issues elsewhere that need to be addressed.
         */
        this.status = {
            isOnline: true,
            isHealthy: true,
            browserInformation: {},
            socket: {},
            resources: {},
        };
        this.onSystemHealthChanged = (event) => {
            this.status = event.detail;
            this.resourceList = Object.values(this.status.resources).sort((a, b) => Number(b.isConnected) - Number(a.isConnected));
        };
        this.onOpen = () => {
            this.isOpen = true;
            this.isDismissed = false;
        };
        this.onClose = () => {
            this.isOpen = false;
            this.isDismissed = false;
        };
        this.onDismiss = () => {
            /**
             * Close if the system is healthy, or if this is the second time the user has dismissed the unhealthy overlay.
             */
            if (this.isDismissed || this.status.isHealthy) {
                this.onClose();
                return;
            }
            this.isDismissed = true;
        };
        this.onUpdateStatus = () => {
            this.status.runAllChecks();
        };
        this.onReportStatus = () => {
            this.status.reportStabilityToGenesis();
        };
        this.onReloadWindow = () => {
            location.reload();
        };
        this.onDocumentKeydown = (e) => {
            if (!e.defaultPrevented) {
                switch (e.key) {
                    case ArrowKeys.ArrowUp:
                        if (e.ctrlKey && e.altKey && !this.isOpen) {
                            this.onOpen();
                            e.preventDefault();
                        }
                        break;
                    case ArrowKeys.ArrowDown:
                        if (e.ctrlKey && e.altKey && this.isOpen) {
                            this.onClose();
                            e.preventDefault();
                        }
                        break;
                    case keyEscape:
                        if (this.isOpen) {
                            this.onDismiss();
                            e.preventDefault();
                        }
                        break;
                }
            }
        };
    }
    isOpenChanged() {
        this.updateTemplateClassNames();
    }
    isDismissedChanged() {
        this.updateTemplateClassNames();
    }
    modeChanged() {
        if (!this.$fastController.isConnected) {
            return;
        }
        if (this.mode === SystemHealthOverlayMode.component) {
            logger.warn('SystemHealthOverlay has support for app overlays only right now.');
        }
    }
    logLevelChanged() {
        networkLogger.level = this.logLevel;
    }
    statusChanged(prev, next) {
        this.updateClassNames();
        if (!this.$fastController.isConnected) {
            return;
        }
        if ((prev === null || prev === void 0 ? void 0 : prev.isHealthy) && (next === null || next === void 0 ? void 0 : next.isHealthy) === false) {
            this.onOpen();
        }
        if ((prev === null || prev === void 0 ? void 0 : prev.isHealthy) === false && (next === null || next === void 0 ? void 0 : next.isHealthy)) {
            this.onClose();
        }
        this.startTimeTillNextUpdateCountdown();
    }
    updateClassNames() {
        this.isOnlineClassNames = classNames(['healthy', this.status.isOnline], ['unhealthy', !this.status.isOnline]);
        this.isHealthyClassNames = classNames(['healthy', this.status.isHealthy], ['unhealthy', !this.status.isHealthy]);
        this.isSocketHealthyClassNames = classNames(['healthy', this.isSocketHealthy], ['unhealthy', !this.isSocketHealthy]);
        this.resourcesAllConnectedClassNames = classNames(['healthy', this.status.resourcesAllConnected], ['unhealthy', !this.status.resourcesAllConnected], ['hidden', !this.validResourceList]);
        this.updateTemplateClassNames();
    }
    updateTemplateClassNames() {
        this.templateClassNames = classNames(['open', this.isOpen], ['dismissed', this.isDismissed]);
        this.containerClassNames = classNames(this.isHealthyClassNames, ['open', this.isOpen], ['dismissed', this.isDismissed]);
    }
    startTimeTillNextUpdateCountdown() {
        const oneSecond = 1000;
        const totalMs = this.status.nextUpdate - Date.now();
        let remainMs = totalMs;
        let remainPercent = 100;
        clearInterval(this.timeTillNextUpdateInterval);
        this.timeTillNextUpdate = {
            totalMs,
            remainMs,
            remainPercent,
        };
        this.timeTillNextUpdateInterval = setInterval(() => {
            remainMs -= oneSecond;
            remainPercent = Math.floor((remainMs / totalMs) * 100);
            this.timeTillNextUpdate = {
                totalMs,
                remainMs,
                remainPercent,
            };
            if (remainMs < 0) {
                clearInterval(this.timeTillNextUpdateInterval);
            }
        }, oneSecond);
    }
    connectedCallback() {
        super.connectedCallback();
        this.networkMonitor.create();
        this.addEventListeners();
    }
    disconnectedCallback() {
        super.disconnectedCallback();
        this.networkMonitor.destroy();
        this.removeEventListeners();
    }
    get isSocketHealthy() {
        return (this.status.isOnline &&
            (!this.status.socket.isConfigured ||
                (this.status.socket.isConfigured && this.status.socket.isConnected)));
    }
    get validResourceList() {
        var _a;
        return ((_a = this.resourceList) === null || _a === void 0 ? void 0 : _a.length) > 0;
    }
    /**
     * @internal
     */
    eventBridge() {
        var _a;
        return (_a = this.config.eventEmitter) !== null && _a !== void 0 ? _a : document;
    }
    /**
     * @internal
     */
    addEventListeners() {
        this.eventBridge().addEventListener(SystemHealthChanged, this.onSystemHealthChanged, true);
        document.addEventListener('keydown', this.onDocumentKeydown);
    }
    /**
     * @internal
     */
    removeEventListeners() {
        this.eventBridge().removeEventListener(SystemHealthChanged, this.onSystemHealthChanged, true);
        document.removeEventListener('keydown', this.onDocumentKeydown);
    }
    static getCheckmarkTemplate(isGood) {
        if (isGood) {
            return html `
        <span class="checkmark">&#x2714;</span>
      `;
        }
        return html `
      <span class="checkmark">&#10008;</span>
    `;
    }
}
__decorate([
    NetworkMonitorConfig
], SystemHealthOverlay.prototype, "config", void 0);
__decorate([
    NetworkMonitor
], SystemHealthOverlay.prototype, "networkMonitor", void 0);
__decorate([
    observable
], SystemHealthOverlay.prototype, "resourceList", void 0);
__decorate([
    observable
], SystemHealthOverlay.prototype, "isOpen", void 0);
__decorate([
    observable
], SystemHealthOverlay.prototype, "isDismissed", void 0);
__decorate([
    observable
], SystemHealthOverlay.prototype, "templateClassNames", void 0);
__decorate([
    observable
], SystemHealthOverlay.prototype, "containerClassNames", void 0);
__decorate([
    observable
], SystemHealthOverlay.prototype, "isOnlineClassNames", void 0);
__decorate([
    observable
], SystemHealthOverlay.prototype, "isHealthyClassNames", void 0);
__decorate([
    observable
], SystemHealthOverlay.prototype, "isSocketHealthyClassNames", void 0);
__decorate([
    observable
], SystemHealthOverlay.prototype, "resourcesAllConnectedClassNames", void 0);
__decorate([
    observable
], SystemHealthOverlay.prototype, "timeTillNextUpdate", void 0);
__decorate([
    observable
], SystemHealthOverlay.prototype, "saveDataCallback", void 0);
__decorate([
    attr
], SystemHealthOverlay.prototype, "mode", void 0);
__decorate([
    attr({ attribute: 'log-level', converter: nullableNumberConverter })
], SystemHealthOverlay.prototype, "logLevel", void 0);
__decorate([
    observable
], SystemHealthOverlay.prototype, "status", void 0);
/**
 * Foundation system health overlay template
 *
 * @public
 */
export const foundationSystemHealthOverlayTemplate = (context, definition) => html `
  <template class="${(x) => x.templateClassNames}">
    <header class="${(x) => x.isHealthyClassNames}" part="header">
      <h1>System ${(x) => pascalCase(x.isHealthyClassNames)}</h1>
      ${when((x) => !x.status.isHealthy, html `
          <span class="data-warning">Data Validity Unknown</span>
        `)}
      <div class="controls">
        ${when((x) => x.isDismissed, html `
            <a href="#" class="open-button" @click="${(x) => x.onOpen()}">View Status</a>
          `)}
        ${when((x) => !x.isDismissed, html `
            <a href="#" class="update-status-button" @click="${(x) => x.onUpdateStatus()}">
              Update Status
            </a>
            <a href="#" class="report-status-button" @click="${(x) => x.onReportStatus()}">
              Report Status
            </a>
          `)}
        ${when((x) => !x.status.isHealthy && x.saveDataCallback, html `
            <a href="#" class="save-data-button" @click="${(x) => x.saveDataCallback()}">
              Save Data
            </a>
          `)}
        <a href="#" class="reload-window-button" @click="${(x) => x.onReloadWindow()}">
          Reload Window
        </a>
        <a href="#" class="dismiss-button" @click="${(x) => x.onDismiss()}">Dismiss (Esc)</a>
      </div>
    </header>
    <div class="container ${(x) => x.containerClassNames}" part="container">
      <section class="network-section ${(x) => x.isOnlineClassNames}" part="network-section">
        <h2>Network ${(x) => pascalCase(x.status.isOnline ? 'online' : 'offline')}</h2>
        <dl class="resource-list">
          <div class="resource ${(x) => x.isOnlineClassNames}">
            <div class="attributes">
              <div class="online">
                <dt>Online</dt>
                <dd>${(x) => x.status.isOnline}</dd>
              </div>
              <div class="authenticated">
                <dt>Authenticated</dt>
                <dd>${(x) => x.status.isAuthenticated}</dd>
              </div>
              ${when((x) => x.status.username, html `
                  <div class="username">
                    <dt>Username</dt>
                    <dd>${(x) => x.status.username}</dd>
                  </div>
                `)}
              <div class="platform-type">
                <dt>Platform Type</dt>
                <dd>${(x) => { var _a; return pascalCase((_a = x.status.browserInformation.platform) === null || _a === void 0 ? void 0 : _a.type); }}</dd>
              </div>
              <div class="os-name">
                <dt>OS Name</dt>
                <dd>${(x) => { var _a; return pascalCase((_a = x.status.browserInformation.os) === null || _a === void 0 ? void 0 : _a.name); }}</dd>
              </div>
              <div class="os-version">
                <dt>OS Version</dt>
                <dd>${(x) => { var _a; return (_a = x.status.browserInformation.os) === null || _a === void 0 ? void 0 : _a.version; }}</dd>
              </div>
            </div>
            <div class="attributes">
              <div class="browser-name">
                <dt>Browser Name</dt>
                <dd>${(x) => { var _a; return pascalCase((_a = x.status.browserInformation.browser) === null || _a === void 0 ? void 0 : _a.name); }}</dd>
              </div>
              <div class="browser-version">
                <dt>Browser Version</dt>
                <dd>${(x) => { var _a; return (_a = x.status.browserInformation.browser) === null || _a === void 0 ? void 0 : _a.version; }}</dd>
              </div>
              <div class="first-updated">
                <dt>First updated</dt>
                <dd>${(x) => formatDateTimeTimestamp(x.status.firstUpdated)}</dd>
              </div>
              <div class="last-updated">
                <dt>Last updated</dt>
                <dd>${(x) => formatDateTimeTimestamp(x.status.lastUpdated)}</dd>
              </div>
              ${when((x) => !x.status.socket.reconnectionLimitReached, html `
                  <div class="next-update">
                    <dt>Next update</dt>
                    <dd>${(x) => formatDateTimeTimestamp(x.status.nextUpdate)}</dd>
                  </div>
                  <div class="next-update-progress">
                    <dt></dt>
                    <dd>
                      <next-update-progress
                        value="${(x) => x.timeTillNextUpdate.remainPercent}"
                      ></next-update-progress>
                    </dd>
                  </div>
                `)}
            </div>
            <div class="controls">
              ${(x) => SystemHealthOverlay.getCheckmarkTemplate(x.status.isOnline)}
            </div>
          </div>
        </dl>
      </section>
      <section class="socket-section ${(x) => x.isSocketHealthyClassNames}" part="socket-section">
        <h2>WebSocket ${(x) => pascalCase(x.isSocketHealthyClassNames)}</h2>
        <dl class="resource-list">
          <div class="resource ${(x) => x.isSocketHealthyClassNames}">
            <div class="attributes">
              <div class="configured">
                <dt>Configured</dt>
                <dd>${(x) => x.status.socket.isConfigured}</dd>
              </div>
              <div class="connected">
                <dt>Connected</dt>
                <dd>${(x) => x.status.socket.isConnected}</dd>
              </div>
              <div class="reconnecting">
                <dt>Reconnecting</dt>
                <dd>${(x) => x.status.socket.isReconnecting}</dd>
              </div>
              <div class="reconnection-exhausted">
                <dt>Reconnection exhausted</dt>
                <dd>${(x) => x.status.socket.reconnectionLimitReached}</dd>
              </div>
            </div>
            <div class="attributes">
              <div class="disconnected-by-server">
                <dt>Disconnected by server</dt>
                <dd>${(x) => x.status.socket.isDisconnectedByServer}</dd>
              </div>
              <div class="closed-clean">
                <dt>Closed clean</dt>
                <dd>${(x) => x.status.socket.closedClean}</dd>
              </div>
              <div class="closed-code">
                <dt>Closed code</dt>
                <dd>${(x) => x.status.socket.closedCode}</dd>
              </div>
              <div class="closed-reason">
                <dt>Closed reason</dt>
                <dd>${(x) => x.status.socket.closedReason}</dd>
              </div>
            </div>
            <div class="controls">
              ${(x) => SystemHealthOverlay.getCheckmarkTemplate(x.isSocketHealthy)}
            </div>
          </div>
        </dl>
      </section>
      <section
        class="resource-section ${(x) => x.resourcesAllConnectedClassNames}"
        part="resource-section"
      >
        <h2>Resources ${(x) => pascalCase(x.resourcesAllConnectedClassNames)}</h2>
        <dl class="resource-list">
          ${repeat((x) => x.resourceList, html `
              <div class="resource ${(x) => (x.isConnected ? 'healthy' : 'unhealthy')}">
                <div class="attributes">
                  <div class="name">
                    <dt>Name</dt>
                    <dd>${(x) => x.name}</dd>
                  </div>
                  <div class="status">
                    <dt>Status</dt>
                    <dd>${(x) => (x.isConnected ? 'Healthy' : 'Unhealthy')}</dd>
                  </div>
                  <div class="last-checked">
                    <dt>Checked</dt>
                    <dd>${(x) => formatDateTimeTimestamp(x.lastUpdated)}</dd>
                  </div>
                  <div class="subscriptions">
                    <dt>Subscriptions</dt>
                    <dd>${(x) => { var _a; return (_a = x.sourceRef) === null || _a === void 0 ? void 0 : _a.length; }}</dd>
                  </div>
                  <div class="source-refs">
                    <dt>Source Refs</dt>
                    <dd>${(x) => x.sourceRef}</dd>
                  </div>
                </div>
                <div class="controls">
                  ${(x) => SystemHealthOverlay.getCheckmarkTemplate(x.isConnected)}
                </div>
              </div>
            `)}
        </dl>
      </section>
    </div>
  </template>
`;
/**
 * Foundation system health overlay styles
 * @public
 */
export const foundationSystemHealthOverlayStyles = (context, definition) => css `
  :host {
    --font-family: Menlo, Consolas, monospace;

    --color-background: black;
    --color-healthy: green;
    --color-unhealthy: red;

    --opacity-low-percent: 10%;
    --opacity-med-percent: 50%;
    --opacity-high-percent: 90%;

    --color-background-opacity-high: rgba(0, 0, 0, 0.9);

    --color-healthy-opacity-low: rgba(0, 128, 0, 0.1);
    --color-healthy-opacity-20: rgba(0, 128, 0, 0.2);
    --color-healthy-opacity-med: rgba(0, 128, 0, 0.5);

    --color-unhealthy-opacity-low: rgba(255, 0, 0, 0.1);
    --color-unhealthy-opacity-20: rgba(255, 0, 0, 0.2);
    --color-unhealthy-opacity-med: rgba(255, 0, 0, 0.5);

    display: none;
    position: fixed;
    inset: 0;
    width: 100vw;
    height: 100vh;
    border: none;
    z-index: 2147483647;
    box-sizing: border-box;
    padding: 2rem 2rem 4rem;
    overflow: auto;
    background-color: var(--color-background-opacity-high);
    line-height: 1.5;
    font-size: 1rem;
    font-family: var(--font-family, Menlo, Consolas, monospace) !important;
  }

  @supports (color: color-mix(in srgb, black, black)) {
    :host {
      --color-background-opacity-high: color-mix(
        in srgb,
        var(--color-background) var(--opacity-high-percent),
        transparent
      );

      --color-healthy-opacity-low: color-mix(
        in srgb,
        var(--color-healthy) var(--opacity-low-percent),
        transparent
      );

      --color-healthy-opacity-20: color-mix(in srgb, var(--color-healthy) 20%, transparent);

      --color-healthy-opacity-med: color-mix(
        in srgb,
        var(--color-healthy) var(--opacity-med-percent),
        transparent
      );

      --color-unhealthy-opacity-low: color-mix(
        in srgb,
        var(--color-unhealthy) var(--opacity-low-percent),
        transparent
      );

      --color-unhealthy-opacity-20: color-mix(in srgb, var(--color-unhealthy) 20%, transparent);

      --color-unhealthy-opacity-med: color-mix(
        in srgb,
        var(--color-unhealthy) var(--opacity-med-percent),
        transparent
      );
    }
  }

  :host(.open.dismissed) {
    display: block;
    overflow: hidden;
    padding: 0;
    height: fit-content;
    min-height: 45px;
    background-color: var(--color-unhealthy-opacity-low);
    inset: unset;
    position: fixed;
    bottom: 0;
  }

  :host(.open.dismissed) header {
    padding: 0 1em;
  }

  :host(.open) {
    display: block;
    height: 100vh;
  }

  .container {
    padding: 0 1rem 1.5rem;
  }

  .container.healthy {
    background-color: var(--color-healthy-opacity-low);
  }

  .container.unhealthy {
    background-color: var(--color-unhealthy-opacity-low);
  }

  .container.open.dismissed {
    display: none;
  }

  header {
    display: flex;
    justify-content: space-between;
    align-items: baseline;
    margin-bottom: 1px;
    padding: 0.5em 1em;
    flex-wrap: wrap;
    flex-direction: row;
  }

  header h1 {
    font-size: 1em;
    font-weight: normal;
  }

  header .controls a {
    font-size: small;
    padding: 0 1em 0 0;
  }

  header.healthy {
    color: var(--color-healthy);
    background-color: var(--color-healthy-opacity-low);
  }

  header.unhealthy {
    color: var(--color-unhealthy);
    background-color: var(--color-unhealthy-opacity-low);
  }

  section {
    padding-top: 1.5em;
    font-size: x-small;
  }

  section h2 {
    font-size: 1.3em;
    margin: 0 0 1em 0;
    text-align: center;
    color: var(--color-unhealthy);
  }

  section.unhealthy h2 {
    color: var(--color-unhealthy);
  }

  section.healthy h2 {
    color: var(--color-healthy);
  }

  .hidden {
    display: none;
  }

  .resource-list {
    display: grid;
    grid-template-columns: repeat(auto-fill, minmax(min(100%, 20rem), 1fr));
    gap: 1rem;
  }

  .resource {
    padding: 0.5em 1em;
    display: grid;
    grid-template-columns: 3fr 1fr;
    border-radius: calc(var(--control-corner-radius) * 1px);
  }

  .resource.healthy {
    border: var(--color-healthy-opacity-med) 1px solid;
    color: var(--color-healthy);
  }

  .resource.unhealthy {
    border: var(--color-unhealthy-opacity-med) 1px solid;
    color: var(--color-unhealthy);
  }

  .resource .attributes > div {
    display: grid;
    grid-template-columns: 1fr 2fr;
  }

  .resource dt {
    color: inherit;
    opacity: 0.7;
  }

  .resource dt:not(:empty)::after {
    content: ': ';
  }

  .resource dd {
    margin: 0 0 0 0.1em;
  }

  .resource .controls {
    display: grid;
    justify-content: center;
    align-items: center;
    border-left: 1px var(--color-background-opacity-high) solid;
  }

  .controls a {
    display: inline-block;
    color: inherit;
    text-underline-offset: 0.2em;
  }

  .controls .checkmark {
    font-size: large;
  }

  .resource.healthy .controls {
    border-left: 1px var(--color-healthy-opacity-20) solid;
  }

  .resource.unhealthy .controls {
    border-left: 1px var(--color-unhealthy-opacity-20) solid;
  }

  .network-section .resource-list,
  .socket-section .resource-list {
    display: grid;
    grid-template-columns: 1fr;
  }

  .network-section .resource,
  .socket-section .resource {
    grid-template-columns: 2fr 2fr 1fr;
  }

  next-update-progress {
    height: 1px;
    margin: 0 1em 0 0;
  }

  .resource.healthy next-update-progress::part(determinate) {
    background-color: var(--color-healthy);
  }

  .resource.healthy next-update-progress::part(progress) {
    background-color: var(--color-healthy-opacity-20);
  }

  .resource.unhealthy next-update-progress::part(determinate) {
    background-color: var(--color-unhealthy);
  }

  .resource.unhealthy next-update-progress::part(progress) {
    background-color: var(--color-unhealthy-opacity-20);
  }

  .data-warning {
    font-size: small;
    margin: 0 0.5em;
    padding: 0.2em 0.5em;
    background-color: var(--color-unhealthy-opacity-20);
  }
`;
/**
 * Foundation system health overlay styles
 * @public
 */
export const foundationSystemHealthOverlayShadowOptions = undefined;
/**
 * Foundation system health overlay styles
 * @public
 */
export const defaultSystemHealthOverlayConfig = {};
/**
 * Foundation system health overlay
 *
 * @public
 * @remarks
 * HTML Element: \<foundation-system-health-overlay\>
 */
export const foundationSystemHealthOverlay = SystemHealthOverlay.compose(Object.assign({ baseName: 'system-health-overlay', template: foundationSystemHealthOverlayTemplate, styles: foundationSystemHealthOverlayStyles, shadowOptions: foundationSystemHealthOverlayShadowOptions }, defaultSystemHealthOverlayConfig));
