import { __awaiter, __decorate } from "tslib";
import { attr, DOM, observable } from '@microsoft/fast-element';
import { FoundationElement } from '@microsoft/fast-foundation';
import { Position } from '../multiselect';
import { foundationSearchBarStyles as styles } from './search-bar.styles';
import { foundationSearchBarTemplate as template } from './search-bar.template';
import { INPUT_MIN_LENGTH, SCROLL_TIMEOUT, } from './search-bar.types';
/**
 * @tagname %%prefix%%-search-bar
 */
export class SearchBar extends FoundationElement {
    constructor() {
        super(...arguments);
        this.searchTerm = '';
        this.appearance = 'neutral';
        this.minlength = INPUT_MIN_LENGTH;
        this.position = Position.BELOW;
        this.placeholder = 'Start typing to search';
        this.selectedOptions = [];
        this.normalizeDelimiter = (inputString) => {
            return inputString.trim().replace(/\t|\r\n|\n|\r|,/g, ' ');
        };
        this.addOrReplaceSelectedText = (previousText, selectedText, newText) => {
            return (selectedText === null || selectedText === void 0 ? void 0 : selectedText.length) > 0
                ? previousText.replace(selectedText, newText)
                : previousText.concat(newText);
        };
    }
    clear() {
        this.selectedOptions = [];
        this.combobox.currentValue = '';
        this.$emit('selectionChange', []);
    }
    clearSearch() {
        this.clear();
    }
    inputdeleted() {
        if (this.selectedOptions.length > 0) {
            this.unselectOption(this.selectedOptions[this.selectedOptions.length - 1]);
        }
    }
    changeHandler() {
        const newOptions = [];
        this.combobox.selectedOptions.forEach((option) => {
            var _a, _b, _c;
            const field = option.value;
            const trimSearchTerm = this.searchTerm.trim();
            newOptions.push({
                field,
                term: trimSearchTerm,
                label: ((_a = this.options
                    .find((availableOption) => availableOption.field === field)) === null || _a === void 0 ? void 0 : _a.label(trimSearchTerm)) || option.text,
                createCriteria: (_b = this.options.find((availableOption) => availableOption.field === field)) === null || _b === void 0 ? void 0 : _b.createCriteria,
                resourceType: (_c = this.options.find((availableOption) => availableOption.field === field)) === null || _c === void 0 ? void 0 : _c.resourceType,
            });
        });
        this.selectedOptions = [...this.selectedOptions, ...newOptions];
        this.selectedOptions = this.selectedOptions.filter((option, index, self) => index ===
            self.findIndex((compareOption) => option.field === compareOption.field && option.term === compareOption.term));
        this.combobox.value = '';
        this.combobox.open = false;
        this.$emit('selectionChange', this.selectedOptions);
        DOM.queueUpdate(() => {
            setTimeout(() => {
                this.selectedOptionsDiv.scrollTo(this.selectedOptionsDiv.scrollWidth, 0);
            }, SCROLL_TIMEOUT);
        });
    }
    pasteHandler(event) {
        var _a;
        this.loading = true;
        const pastedText = event.clipboardData.getData('text/plain');
        const formattedText = this.normalizeDelimiter(pastedText);
        const selectedText = (_a = window.getSelection()) === null || _a === void 0 ? void 0 : _a.toString();
        const newText = this.addOrReplaceSelectedText(this.combobox.value, selectedText, formattedText);
        if ((newText === null || newText === void 0 ? void 0 : newText.length) >= this.minlength) {
            this.combobox.value = newText;
            this.searchTerm = newText;
            this.combobox.open = true;
        }
        this.loading = false;
    }
    hasSelectedOption(term, field) {
        return this.selectedOptions.some((option) => {
            return option.term === term && option.field === field;
        });
    }
    unselectOption(option) {
        this.selectedOptions = this.selectedOptions.filter((x) => x !== option);
        this.$emit('selectionChange', this.selectedOptions);
    }
    inputHandler() {
        return __awaiter(this, void 0, void 0, function* () {
            this.loading = true;
            this.searchTerm = this.combobox.value;
            if (this.combobox.value.length >= this.minlength) {
                this.filterSelectedOptions(this.combobox.value);
            }
            this.loading = false;
        });
    }
    filterSelectedOptions(term) {
        this.filteredOptions = this.options.filter((option) => !this.selectedOptions.find((selectedOption) => selectedOption.field === option.field && selectedOption.term.trim() === term.trim()));
    }
}
__decorate([
    observable
], SearchBar.prototype, "defaultSlottedNodes", void 0);
__decorate([
    observable
], SearchBar.prototype, "options", void 0);
__decorate([
    observable
], SearchBar.prototype, "searchTerm", void 0);
__decorate([
    attr
], SearchBar.prototype, "appearance", void 0);
__decorate([
    attr
], SearchBar.prototype, "minlength", void 0);
__decorate([
    attr
], SearchBar.prototype, "position", void 0);
__decorate([
    attr
], SearchBar.prototype, "placeholder", void 0);
__decorate([
    attr
], SearchBar.prototype, "disabled", void 0);
__decorate([
    observable
], SearchBar.prototype, "loading", void 0);
__decorate([
    observable
], SearchBar.prototype, "selectedOptions", void 0);
__decorate([
    observable
], SearchBar.prototype, "filteredOptions", void 0);
// explicitly declaring for extensibility
export const foundationSearchBarShadowOptions = undefined;
export const defaultSearchBarConfig = {};
/**
 * The Foundation Search Bar
 *
 * @public
 * @remarks
 * HTML Element: \<foundation-search-bar\>
 */
export const foundationSearchBar = SearchBar.compose(Object.assign({ baseName: 'search-bar', template,
    styles, shadowOptions: foundationSearchBarShadowOptions }, defaultSearchBarConfig));
